/*
 * encoder.cpp
 *
 *  Created on: 2010/09/14
 *      Author: shu
 */

#include "encoder.h"
#include "sequence_type.h"
#include <stdint.h>
#include <string.h>
#include <string>
#include <algorithm>

using namespace std;

Encoder::Encoder(const SequenceType &type)
: min_code_(0), max_code_(0) {

  string letters("");

  letters += type.GetRegularLetters();
  letters += type.GetAmbiguousLetters();
  transform(letters.begin(), letters.end(), letters.begin(), ToUpper());

  min_regular_letter_code_ = 0;
  max_regular_letter_code_ = static_cast<Code>(type.GetRegularLetters().length() - 1);
  unknown_code_ = static_cast<Code>(letters.length());
  unknown_letter = tolower(type.GetUnknownLetter());
  max_code_ = unknown_code_;
  for (int i = 0; i < UCHAR_MAX; ++i) {
    codes_[i] = unknown_code_;
  }
  Code code = 0;

  for (uint32_t i = 0; i < letters.length(); ++i, ++code) {
    codes_[static_cast<int>(letters[i])] = code;
    codes_[tolower(letters[i])] = code;
  }
}

Code Encoder::Encode(char letter) const {
  return codes_[static_cast<int>(letter)];
}

void Encoder::Encode (const string &sequence_data, Code *encoded_sequence) const {
  for (uint32_t i = 0; i < sequence_data.length(); ++ i) {
    encoded_sequence[i] = codes_[static_cast<int>(sequence_data[i])];
  }
}

bool Encoder::IsUnknown(char letter) const {
  char l = tolower(letter);
  return (codes_[static_cast<int>(l)] == unknown_code_) && (l != unknown_letter);
}
